/*
    pinco - a didactic framework for Web development
    Copyright (C) 2002-2003  Giansalvatore Mecca - mecca@unibas.it
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ----
    pinco - un framework didattico per lo sviluppo Web
    Copyright (C) 2002-2003  Giansalvatore Mecca - mecca@unibas.it
 
    Questo  programma   software  libero;   lecito redistribuirlo  o
    modificarlo secondo i termini  della Licenza Pubblica Generica GNU
    come  pubblicata dalla Free  Software Foundation; o la versione 2
    della licenza o (a propria scelta) una versione successiva.
 
    Questo programma   distribuito nella  speranza che sia  utile, ma
    SENZA  ALCUNA GARANZIA;  senza  neppure la  garanzia implicita  di
    NEGOZIABILIT  o di  APPLICABILIT PER  UN PARTICOLARE  SCOPO.  Si
    veda la Licenza Pubblica Generica GNU per avere maggiori dettagli.
 
    Questo  programma deve  essere  distribuito assieme  ad una  copia
    della Licenza Pubblica Generica GNU;  in caso contrario, se ne pu
    ottenere  una scrivendo  alla Free  Software Foundation,  Inc., 59
    Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
 */

package it.unibas.pinco.framework;

import it.unibas.pinco.azioni.Azione;
import it.unibas.pinco.azioni.AzioneIniziale;
import it.unibas.pinco.azioni.AzioneFinale;

public class CommandFactory {
    
    private java.util.HashMap cacheComandi = new java.util.HashMap();
    private static CommandFactory singleton = new CommandFactory();
    
    private CommandFactory() {}
    
    public static CommandFactory getInstance() {
        return singleton;
    }
    
    public Azione getAzione(String nomeAzione) throws PincoException {
        Azione azione = (Azione)cacheComandi.get(nomeAzione);
        if (azione == null) {
            try {
                azione = (Azione)Class.forName(nomeAzione).newInstance();
            } catch (ClassNotFoundException e) {
                Logger.logMessage(e.toString());
                throw new PincoException("La classe corrispondente all'azione non e' disponibile " + e);
            } catch (InstantiationException e) {
                Logger.logMessage(e.toString());
                throw new PincoException("Impossibile istanziare l'azione" + e);
            } catch (IllegalAccessException e) {
                Logger.logMessage(e.toString());
                throw new PincoException("Accesso illegale alla classe di azione" + e);
            }
            cacheComandi.put(nomeAzione, azione);
            Logger.logMessage("CommandFactory +++ " + nomeAzione + " trovato");
        }
        return azione;
    }
    
    public AzioneIniziale getAzioneIniziale(String nomeAzione) throws PincoException {
        AzioneIniziale azioneIniziale = null;
        try {
            azioneIniziale = (AzioneIniziale)Class.forName(nomeAzione).newInstance();
        } catch (ClassNotFoundException e) {
            Logger.logMessage(e.toString());
            throw new PincoException("La classe corrispondente all'azione non e' disponibile " + e);
        } catch (InstantiationException e) {
            Logger.logMessage(e.toString());
            throw new PincoException("Impossibile istanziare l'azione" + e);
        } catch (IllegalAccessException e) {
            Logger.logMessage(e.toString());
            throw new PincoException("Accesso illegale alla classe di azione" + e);
        }
        Logger.logMessage("CommandFactory +++ " + nomeAzione + " trovato");
        return azioneIniziale;
    }

    public AzioneFinale getAzioneFinale(String nomeAzione) throws PincoException {
        AzioneFinale azioneFinale = null;
        try {
            azioneFinale = (AzioneFinale)Class.forName(nomeAzione).newInstance();
        } catch (ClassNotFoundException e) {
            Logger.logMessage(e.toString());
            throw new PincoException("La classe corrispondente all'azione non e' disponibile " + e);
        } catch (InstantiationException e) {
            Logger.logMessage(e.toString());
            throw new PincoException("Impossibile istanziare l'azione" + e);
        } catch (IllegalAccessException e) {
            Logger.logMessage(e.toString());
            throw new PincoException("Accesso illegale alla classe di azione" + e);
        }
        Logger.logMessage("CommandFactory +++ " + nomeAzione + " trovato");
        return azioneFinale;
    }
}
