/*
    pinco - a didactic framework for Web development
    Copyright (C) 2002-2003  Giansalvatore Mecca - mecca@unibas.it
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ----
    pinco - un framework didattico per lo sviluppo Web
    Copyright (C) 2002-2003  Giansalvatore Mecca - mecca@unibas.it
 
    Questo  programma   software  libero;   lecito redistribuirlo  o
    modificarlo secondo i termini  della Licenza Pubblica Generica GNU
    come  pubblicata dalla Free  Software Foundation; o la versione 2
    della licenza o (a propria scelta) una versione successiva.
 
    Questo programma   distribuito nella  speranza che sia  utile, ma
    SENZA  ALCUNA GARANZIA;  senza  neppure la  garanzia implicita  di
    NEGOZIABILIT  o di  APPLICABILIT PER  UN PARTICOLARE  SCOPO.  Si
    veda la Licenza Pubblica Generica GNU per avere maggiori dettagli.
 
    Questo  programma deve  essere  distribuito assieme  ad una  copia
    della Licenza Pubblica Generica GNU;  in caso contrario, se ne pu
    ottenere  una scrivendo  alla Free  Software Foundation,  Inc., 59
    Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
 */

package it.unibas.pinco.framework;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import it.unibas.pinco.azioni.Azione;
import it.unibas.pinco.azioni.AzioneIniziale;
import it.unibas.pinco.azioni.AzioneFinale;

public class Controllo extends HttpServlet {
    
    private MappaDelleAzioni mappaDelleAzioni;
    private boolean inizializzato;
    private static final String VERSIONE = "v. 0.8";
    private static final String NOME_FILE_AZIONI = "/WEB-INF/azioni-pinco.xml";
    
    public void init() throws ServletException {
        ServletContext servletContext = this.getServletContext();
        caricaMappa(servletContext);
        eseguiAzioneIniziale(servletContext);
        this.inizializzato = true;
    }
    
    private void caricaMappa(ServletContext servletContext) throws PincoException {
        String nomeFileAzioni = this.getNomeFileAzioni(servletContext);
        String percorsoFileAzioni = servletContext.getRealPath(nomeFileAzioni);
        this.mappaDelleAzioni = new MappaDelleAzioni();
        this.mappaDelleAzioni.carica(percorsoFileAzioni);
    }
    
    private String getNomeFileAzioni(ServletContext servletContext) {
        ServletConfig servletConfig = getServletConfig();
        String nomeFileAzioni = servletConfig.getInitParameter("azioni-pinco");
        if (nomeFileAzioni == null) {
            nomeFileAzioni = Controllo.NOME_FILE_AZIONI;
        }
        return nomeFileAzioni;
    }
    
    private void eseguiAzioneIniziale(ServletContext servletContext) throws PincoException {
        synchronized(servletContext) {
            Object pincoInit = servletContext.getAttribute("pincoInit");
            if (pincoInit != null) {
                Logger.logMessage("\npinco "+ VERSIONE +" +++ azione iniziale gia' eseguita");
                return;
            }
            String nomeAzioneIniziale = this.mappaDelleAzioni.getAzioneIniziale();
            if (nomeAzioneIniziale == null) {
                Logger.logMessage("\npinco " + VERSIONE + " +++ azione iniziale inesistente");
                return;
            }
            Logger.logMessage("\npinco " + VERSIONE + " +++ azione iniziale = " + nomeAzioneIniziale);
            AzioneIniziale azioneIniziale = null;
            azioneIniziale = (AzioneIniziale)CommandFactory.getInstance().getAzioneIniziale(nomeAzioneIniziale);
            azioneIniziale.esegui(servletContext);
            servletContext.setAttribute("pincoInit", new Object());
            Logger.logMessage("\npinco " + VERSIONE + " +++ controllo inizializzato");
        }
    }
    
    public boolean getInizializzato() {
        return this.inizializzato;
    }
    
    public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
        doPost(request, response);
    }
    
    public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
        if (!this.getInizializzato()) {
            throw new PincoException("Controllo non inizializzato");
        }
        String comando = request.getServletPath().substring(1);
        String nomeClasseAzione = mappaDelleAzioni.getAzione(comando);
        Logger.logMessage("\npinco " + VERSIONE + " +++ comando: " + comando);
        if (nomeClasseAzione == null) {
            throw new PincoException("Comando inesistente");
        }
        Logger.logMessage("pinco " + VERSIONE + " +++ azione:  " + nomeClasseAzione);
        String esito = null;
        Azione azione = CommandFactory.getInstance().getAzione(nomeClasseAzione);
        esito = azione.esegui(request);
        String prossimoSchermo = mappaDelleAzioni.getSchermoSuccessivo(comando, esito);
        Logger.logMessage("pinco " + VERSIONE + " +++ esito:   " + esito);
        if (prossimoSchermo == null) {
            throw new PincoException("Schermo successivo indefinito");
        }
        Logger.logMessage("pinco " + VERSIONE + " +++ schermo: " + prossimoSchermo);
        request.setAttribute("pinco", new Object());
        ServletContext servletContext = getServletContext();
        RequestDispatcher requestDispatcher = servletContext.getRequestDispatcher(prossimoSchermo);
        requestDispatcher.forward(request, response);
        return;
    }
    
    public void destroy() {
        ServletContext servletContext = getServletContext();
        synchronized(servletContext) {
            Object pincoDestroy = servletContext.getAttribute("pincoDestroy");
            if (pincoDestroy != null) {
                Logger.logMessage("\npinco "+ VERSIONE +" +++ azione finale gia' eseguita");
                return;
            }
            String nomeAzioneFinale = this.mappaDelleAzioni.getAzioneFinale();
            if (nomeAzioneFinale == null) {
                Logger.logMessage("\npinco " + VERSIONE + " +++ azione finale inesistente");
                return;
            }
            Logger.logMessage("\npinco " + VERSIONE + " +++ azione finale = " + nomeAzioneFinale);
            AzioneFinale azioneFinale = null;
            try {
                azioneFinale = (AzioneFinale)CommandFactory.getInstance().getAzioneFinale(nomeAzioneFinale);
                azioneFinale.esegui(servletContext);
            } catch (PincoException e) {
                Logger.logMessage(e.toString());
            }
            servletContext.setAttribute("pincoDestroy", new Object());
            Logger.logMessage("\npinco " + VERSIONE + " +++ controllo distrutto");
        }
    }
    
}